import React from 'react';
import * as PropTypes from 'prop-types';

import TextLabel from '../TextLabel/TextLabel';
import IconButton, { ICON_SIZE } from '../IconButton/IconButton';

/**
 * Data Table Column Header with filter controls
 */
const DataTableColumnHeader = (props) => {
    console.log('DATA TABLE COLUMN HEADER PROPS: ', props); // eslint-disable-line

    const {
        column,
        displayName,
        onColumnPinned,
        onFilterRequested,
        showFilters,
    } = props;
    const {
        colDef,
        pinned,
    } = column;
    const {
        field,
    } = colDef;

    // Generate filter button props
    const filterProps = {
        iconName: 'filter_list',
        iconSize: ICON_SIZE.SMALL,
        onClick: () => {
            console.log('DATA TABLE: COLUMN HEADER: FILTER EVENT: ', field); // eslint-disable-line
            if (onFilterRequested) {
                onFilterRequested(field, !showFilters);
            }
        },
        title: `${showFilters ? 'close' : 'open'} sort and filter`,
    };

    // Generate pin button props
    const pinProps = {
        iconName: pinned ? 'lock_open' : 'lock',
        iconSize: ICON_SIZE.SMALL,
        onClick: () => {
            console.log('DATA TABLE: COLUMN HEADER: PIN EVENT: ', field); // eslint-disable-line
            if (onColumnPinned) {
                onColumnPinned(field, !pinned);
            }
        },
        title: pinned ? 'unlock column' : 'lock column',
    };

    return (
        <div style={ {
            display: 'flex',
            justifyContent: 'center',
            alignContent: 'center',
            alignItems: 'center',
        } }>
            <TextLabel>
                { displayName }
            </TextLabel>
            <IconButton { ...pinProps } />
            <IconButton { ...filterProps } />
        </div>
    );
};

DataTableColumnHeader.propTypes = {
    column: PropTypes.object,
    displayName: PropTypes.string.isRequired,
    onColumnPinned: PropTypes.func,
    onFilterRequested: PropTypes.func,
    showFilters: PropTypes.bool,
};

DataTableColumnHeader.defaultProps = {};

export default DataTableColumnHeader;
