import React, { Component } from 'react';
import * as PropTypes from 'prop-types';

import TableCell from '@material-ui/core/TableCell';
import TableRow from '@material-ui/core/TableRow';

class DataTableBodyMUI extends Component {
    shouldComponentUpdate(nextProps, nextState, nextContext) {
        if (nextProps.columns !== this.props.columns) {
            return true;
        }
        if (nextProps.rows !== this.props.rows) {
            return true;
        }
        return false;
    }

    render() {
        console.log('DATA TABLE BODY: RENDER: ', this.props); // eslint-disable-line
        const {
            columns,
            cellRenderer,
            rows,
        } = this.props;

        return (
            rows.map((row, rowIndex) => {
                return (
                    // eslint-disable-next-line react/no-array-index-key
                    <TableRow key={ rowIndex }>
                        {
                            columns.map((column, columnIndex) => {
                                const {
                                    filterable,
                                    filterType,
                                    filterValue,
                                    frozen,
                                    label,
                                    movable,
                                    name,
                                    resizable,
                                    sortable,
                                    sortDescending,
                                    sortAscending,
                                    width,
                                } = column;
                                let cellValue = row[name];
                                if (cellRenderer) {
                                    cellValue = cellRenderer(cellValue, row, rowIndex, column, columnIndex);
                                }
                                if (typeof cellValue === 'boolean') {
                                    cellValue = cellValue.toString();
                                }
                                return (
                                    // eslint-disable-next-line react/no-array-index-key
                                    <TableCell
                                        style={ { whiteSpace: 'nowrap' } }
                                        key={ `${rowIndex}_${columnIndex}_${name}` }>
                                        { cellValue }
                                    </TableCell>
                                );
                            })
                        }
                    </TableRow>
                );
            })
        );
    }
}

export default DataTableBodyMUI;
