import React, { Component } from 'react';
import * as PropTypes from 'prop-types';

import TableCell from '@material-ui/core/TableCell';
import TableRow from '@material-ui/core/TableRow';

import { useDrag, useDrop } from 'react-dnd';

import { ItemTypes } from './constants';

const DataTableHeaderCellMUI = (props) => {
    // console.log('DATA TABLE HEADER CELL: RENDER: ', props); // eslint-disable-line
    const {
        column,
        headerRenderer,
        index,
        onHeaderMoved,
        showFilters,
    } = props;

    const [
        { isDragging },
        drag,
        preview,
    ] = useDrag({
        item: {
            index,
            type: ItemTypes.COLUMN,
        },
        collect: (monitor) => {
            return {
                isDragging: !!monitor.isDragging(),
            };
        },
    });

    const [
        {
            isOver,
            canDrop,
        },
        drop,
    ] = useDrop({
        accept: ItemTypes.COLUMN,
        drop: (item) => {
            if (onHeaderMoved) {
                onHeaderMoved({
                    column,
                    sourceIndex: item.index,
                    targetIndex: index,
                });
            }
        },
        collect: (monitor) => {
            return {
                isOver: !!monitor.isOver(),
                canDrop: !isDragging && !!monitor.canDrop(),
            };
        },
    });

    // console.log('DATA TABLE HEADER CELL: DRAGGING: ', isDragging, index); // eslint-disable-line
    // console.log('DATA TABLE HEADER CELL: IS_OVER: ', isOver, index); // eslint-disable-line
    // console.log('DATA TABLE HEADER CELL: CAN_DROP: ', canDrop, index); // eslint-disable-line

    const {
        filterable,
        filterType,
        filterValue,
        frozen,
        label,
        movable,
        name,
        resizable,
        sortable,
        sortDescending,
        sortAscending,
        width,
    } = column;
    let cellValue = label || name;
    if (headerRenderer) {
        cellValue = headerRenderer(cellValue, column, index);
    }
    return (
        <TableCell
            ref={ drop }
            style={ {
                border: canDrop ? '1px solid black' : 'none',
                backgroundColor: (!isDragging && isOver) ? '#ccc' : 'inherit',
            } }
            key={ name }>
            { /* TODO: make a Chip component for the preview */ }
            <div
                style={ {
                    backgroundColor: '#06f',
                    color: '#fff',
                    borderRadius: '0.25rem',
                    padding: '0.25rem 1rem',
                    position: 'absolute',
                    top: -9999999,
                } }
                ref={ preview }>
                { label || name }
            </div>
            <div
                ref={ drag }
                style={ {
                    opacity: isDragging ? 0.5 : 1,
                } }>
                { cellValue }
            </div>
        </TableCell>
    );
};

export default DataTableHeaderCellMUI;
