import React, { Component } from 'react';
import * as PropTypes from 'prop-types';

import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableHead from '@material-ui/core/TableHead';

import DataTableHeaderRow from './DataTableHeaderRowMUI';
import DataTableFilterRow from './DataTableFilterRowMUI';
import DataTableBody from './DataTableBodyMUI';

import getStyles from './DataTableMUI.styles';
import DataTableHeaderCell from "./DataTableHeaderCellMUI";

const SORT_DIRECTIONS = {
    NONE: '',
    ASC: 'asc',
    DESC: 'desc',
};
const FILTER_TYPES = {
    BOOLEAN: 'BOOLEAN',
    DATE: 'DATE',
    DATETIME: 'DATETIME',
    DECIMAL: 'DECIMAL',
    INTEGER: 'INTEGER',
    STRING: 'STRING',
    TIME: 'TIME',
    YEAR: 'YEAR',
};

// @TODO Need a right click handler
// @TODO Need paging props

class DataTableMUI extends Component {
    constructor(props) {
        super(props);
        const {
            columns,
        } = props;
        this.state = {
            columns,
        };
    }

    render() {
        // const {
        //     cellRenderer,
        //     cols,
        //     contextStyles,
        //     filterableColumns,
        //     floatingFilterComponent: FloatingFilterComponent,
        //     multiSelectMode,
        //     onApplyFilters,
        //     onColumnPinned,
        //     onFilterRequested,
        //     onSortRequested,
        //     resizableColumns,
        //     rows,
        //     showFilters,
        //     sortableColumns,
        //     sortDirection,
        //     suppressMovableColumns,
        //     testID,
        // } = this.props;

        const {
            cellRenderer,
            filterRenderer,
            headerRenderer,
            rowRenderer,
            contextStyles,
            rows,
            showFilters,
            testID,
        } = this.props;
        const {
            columns,
        } = this.state;

        console.log('DATA TABLE: RENDER: ', this.props); // eslint-disable-line

        const styles = getStyles(contextStyles);

        // Generate container props
        const containerProps = {
            style: styles.container,
            className: 'c-data-table__container',
        };
        if (testID) { containerProps['data-test-id'] = `${testID}__data-table__container`; }

        // // Generate viewport props
        // const viewportProps = {
        //     style: styles.viewport,
        //     className: `ag-theme-${THEME} c-data-table__viewport`,
        // };
        // if (testID) { containerProps['data-test-id'] = `${testID}__data-table__viewport`; }
        //
        // if (FloatingFilterComponent) {
        //     console.log('USING CUSTOM FLOATING FILTER' ); // eslint-disable-line
        // }

        // Generate data table props
        // const dataTableProps = {
        //     reactNext: true,
        //     floatingFiltersHeight: 110,
        //     key: this.key,
        //     suppressColumnVirtualisation: true,
        //     rowBuffer: 10,
        //     columnDefs: cols,
        //     defaultColDef: {
        //         cols,
        //         cellRendererFramework: cellRenderer || DataTableCell,
        //         columnStyles: styles.column,
        //         filter: filterableColumns,
        //         floatingFilterComponentFramework: FloatingFilterComponent || DataTableFloatingFilter,
        //         floatingFilterComponentParams: {
        //             filterTypes: FILTER_TYPES,
        //             onApplyFilters: (field, value) => {
        //                 console.log('DATA TABLE: FILTER CHANGED: FIELD: ', field); // eslint-disable-line
        //                 console.log('DATA TABLE: FILTER CHANGED: VALUE: ', value); // eslint-disable-line
        //                 if (onApplyFilters) {
        //                     onApplyFilters(field, value);
        //                 }
        //             },
        //             suppressFilterButton: true,
        //         },
        //         headerComponentFramework: DataTableColumnHeader,
        //         headerComponentParams: {
        //             sortDirection,
        //             sortDirections: SORT_DIRECTIONS,
        //             filterableColumns,
        //             showFilters,
        //             onSortChanged: (field, sortDirection) => {
        //                 console.log('DATA TABLE: SORT CHANGED: FIELD: ', field); // eslint-disable-line
        //                 console.log('DATA TABLE: SORT CHANGED: DIRECTION: ', sortDirection); // eslint-disable-line
        //                 if (onSortRequested) {
        //                     onSortRequested(field, sortDirection);
        //                 }
        //             },
        //             onFilterToggle: (field, showFilter) => {
        //                 console.log('DATA TABLE: FILTER TOGGLE: FIELD: ', field); // eslint-disable-line
        //                 console.log('DATA TABLE: FILTER TOGGLE: SHOW FILTER: ', showFilter); // eslint-disable-line
        //                 if (onFilterRequested) {
        //                     onFilterRequested(field, showFilter);
        //                 }
        //             },
        //             onPinToggle: (field, pin) => {
        //                 console.log('DATA TABLE: PIN CHANGED: FIELD: ', field); // eslint-disable-line
        //                 console.log('DATA TABLE: PIN CHANGED: PIN: ', pin); // eslint-disable-line
        //                 if (onColumnPinned) {
        //                     onColumnPinned(field, pin);
        //                 }
        //             },
        //         },
        //         suppressMenu: true,
        //         suppressMovable: suppressMovableColumns,
        //         resizable: resizableColumns,
        //         sortable: sortableColumns,
        //     },
        //     floatingFilter: filterableColumns && showFilters,
        //     onGridReady: this.handleGridReady,
        //     onRowClicked: this.handleRowClicked,
        //     onRowDoubleClicked: this.handleRowDoubleClicked,
        //     rowData: rows,
        //     rowSelection: multiSelectMode ? 'multiple' : 'single',
        //     suppressCellSelection: true,
        // };

        return (
            <div { ...containerProps }>
                <Table>
                    <TableHead>
                        <DataTableHeaderRow
                            columns={ columns }
                            headerRenderer={ headerRenderer }
                            onHeaderMoved={ this.handleHeaderMoved }
                            showFilters={ showFilters } />
                        <DataTableFilterRow
                            columns={ columns }
                            filterRenderer={ filterRenderer }
                            showFilters={ showFilters } />
                    </TableHead>
                    <TableBody>
                        <DataTableBody
                            columns={ columns }
                            filterRenderer={ filterRenderer }
                            rows={ rows } />
                    </TableBody>
                </Table>
            </div>
        );
    }

    handleHeaderMoved = (params) => {
        const {
            column,
            sourceIndex,
            targetIndex,
        } = params;
        console.log('MOVED COLUMN: ', column, ' FROM:', sourceIndex, ' TO COLUMN:', targetIndex); // eslint-disable-line

        const {
            columns,
        } = this.state;

        // Clone columns array
        const newColumns = [ ...columns ];

        // Reorder columns
        newColumns.splice(
            targetIndex,
            0,
            newColumns.splice(sourceIndex, 1)[0]
        );

        // Need to modify column object due to 'immutable' use by RDG
        // const reorderedColumns = newColumns.map((c, i) => {
        //     return {
        //         ...c,
        //         _reRenderKey: `${c.key}_${i}`,
        //     };
        // });

        // Update columns
        this.setState({
            columns: newColumns,
        });
    };

    /**
     * Renders columns
     * @param {Object} columnList - meta data about columns
     * @returns {Array}
     */
    // renderColumns = (columnList) => (
    //     columnList.map((column) => (
    //         <AgGridColumn
    //             autoHeight
    //             field={ column.field }
    //             filter={ column.filter }
    //             headerName={ column.headerName }
    //             key={ column.field }
    //             pinned={ column.pinned }
    //             resizable={ column.resizable }
    //             sortable={ column.sortable }
    //             suppressMovable={ column.suppressMovable }
    //             width={ column.width } />
    //     ))
    // );

    /** Makes grid api available to component */
    // handleGridReady = (grid) => {
    //     this.api = grid.api;
    // };

    /**
     * Called on click of a row
     * @returns {void}
     */
    // handleRowClicked = () => {
    //     const { onRowClick } = this.props;
    //
    //     if (onRowClick) {
    //         onRowClick({
    //             data: this.api.getSelectedRows(),
    //             nodes: this.api.getSelectedNodes(),
    //         });
    //     }
    // };

    /**
     * Called on double click of a row
     * @returns {void}
     */
    // handleRowDoubleClicked = () => {
    //     const { onRowDoubleClick } = this.props;
    //
    //     if (onRowDoubleClick) {
    //         onRowDoubleClick({
    //             data: this.api.getSelectedRows()[0],
    //             node: this.api.getSelectedNodes()[0],
    //         });
    //     }
    // };
}

DataTableMUI.propTypes = {
    /**
     * Custom cell renderer
     * @param {Object} params column details for the cell
     */
    cellRenderer: PropTypes.func,

    /** Array containing column definition object */
    columns: PropTypes.arrayOf(PropTypes.shape({
        /** Unique column id which maps to row property */
        field: PropTypes.string,

        /** Enable / disable individual filterable column (overrides global) */
        filter: PropTypes.bool,

        /** Column header display text */
        headerName: PropTypes.string,

        /** Enables column to be pinned */
        pinned: PropTypes.bool,

        /** Enable / disable individual resizable column (overrides global) */
        resizable: PropTypes.bool,

        /** Enable / disable individual sortable column (overrides global) */
        sortable: PropTypes.bool,

        /** Enable / disable individual movable column (overrides global) */
        suppressMovable: PropTypes.bool,

        /** Width of column */
        width: PropTypes.number,

        // TODO
        type: PropTypes.oneOf([
            FILTER_TYPES.BOOLEAN,
            FILTER_TYPES.DATE,
            FILTER_TYPES.DATETIME,
            FILTER_TYPES.DECIMAL,
            FILTER_TYPES.INTEGER,
            FILTER_TYPES.STRING,
            FILTER_TYPES.TIME,
        ]),
    })).isRequired,

    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Column level styles */
        column: PropTypes.object,

        /** Style object for container */
        container: PropTypes.object,

        /** Style object for evenRows*/
        evenRows: PropTypes.object,

        /** Style object for grid filter dialog */
        filter: PropTypes.object,

        /** Style object for grid filter dialog container */
        filterContainer: PropTypes.object,

        /** Style object for grid */
        grid: PropTypes.object,

        /** Style object for header cell */
        headerCell: PropTypes.object,

        /** Style object for header text */
        headerText: PropTypes.object,

        /** Style object for oddRows*/
        oddRows: PropTypes.object,

        /** Style object for grid row */
        row: PropTypes.object,

        /** Style object for selected grid row */
        rowSelected: PropTypes.object,
    }),

    /**
     * Enable / disable global filterable columns (floating filter)
     * Shows filter icon buttons on column headers to toggle filter visibility
     */
    filterableColumns: PropTypes.bool,

    /** Custom floating filter */
    floatingFilterComponent: PropTypes.func,

    /** Enables multiple row selection */
    multiSelectMode: PropTypes.bool,

    // TODO: implement to call server for data refresh
    /** Called after application of filters */
    onApplyFilters: PropTypes.func,

    // TODO: implement to save column position after data refresh
    /** Called after moving of columns */
    // onColumnMoved: PropTypes.func,

    // TODO: implement to save column pinned state after data refresh
    /**
     * Called after pinning of columns
     * @param {String} field
     * @param {Boolean} pin
     */
    onColumnPinned: PropTypes.func,

    // TODO: implement to save column resized state after data refresh
    /** Called after resize of columns */
    // onColumnResized: PropTypes.func,

    /**
     * //
     */
    onFilterRequested: PropTypes.func,

    /**
     * Called on row click
     * @param { { data: Array<Object>, nodes: Array<Object> } }
     */
    onRowClick: PropTypes.func,

    /**
     * Called on row double click
     * @param { { data: Object, node: Object } }
     */
    onRowDoubleClick: PropTypes.func,

    /**
     * Called after sort request
     * @param {String} field
     * @param {String} sortMethod
     * */
    onSortRequested: PropTypes.func,

    /** Enable / disable global resizable columns */
    resizableColumns: PropTypes.bool,

    /** Array of objects of column fields */
    rows: PropTypes.arrayOf(PropTypes.object).isRequired,

    /** Enable / disable visibility of the floating filter (if filterableColumns is enabled) */
    showFilters: PropTypes.bool,

    /**
     * Enable / disable global sortable columns
     * Shows sort icon buttons on column headers to request sorting for the field
     */
    sortableColumns: PropTypes.bool,

    /** Current sort directive */
    sortDirection: PropTypes.oneOf([
        SORT_DIRECTIONS.NONE,
        SORT_DIRECTIONS.ASC,
        SORT_DIRECTIONS.DESC,
    ]),

    /** Enable / disable global movable columns */
    suppressMovableColumns: PropTypes.bool,

    /** ID used for testing */
    testID: PropTypes.string,
};

DataTableMUI.defaultProps = {
    sortDirection: '',
};

export default DataTableMUI;
export {
    SORT_DIRECTIONS,
    FILTER_TYPES,
};
