import React, { Component } from 'react';
import * as PropTypes from 'prop-types';
import ReactDataGrid from 'react-data-grid';

// import getStyles from './DataTableRDG.styles';

const {
    DraggableHeader: { DraggableContainer },
} = require('react-data-grid-addons');

// TODO: FIX COLUMN RESIZING - fork and implement https://github.com/adazzle/react-data-grid/pull/1658/commits/d62d5281d12569549ae6874d2f45fd3ae296f67b
// TODO: FIX PREVENT CELL SELECTION - https://github.com/adazzle/react-data-grid/issues/1655
// TODO: FIX FROZEN COLUMNS - trigger column 'resize' after freeze?  reorder before render?
// TODO: FIX INABILITY TO MANUALLY SHOW/HIDE FILTERS - add new prop

class DataTableRDG extends Component {
    static propTypes = {
        columns: PropTypes.array,
        rows: PropTypes.array,
    };

    constructor(props) {
        super(props);
        const {
            columns,
            rows,
        } = props;
        this.state = {
            columns,
            rows,
        };
    }

    render() {
        const {
            columns,
            rows,
        } = this.state;

        console.log('DATA TABLE RDG: RENDER: ', this.props, this.state); // eslint-disable-line

        return (
            <DraggableContainer onHeaderDrop={ this.handleHeaderDrop }>
                <ReactDataGrid
                    cellNavigationMode="none"
                    columns={ columns }
                    enableCellAutoFocus={ false }
                    minHeight={ 500 }
                    ref={ (r) => { this.grid = r; } }
                    rowGetter={ this.rowGetter }
                    rowsCount={ rows.length } />
            </DraggableContainer>
        );
    }

    componentDidUpdate() {
        console.log('DATA TABLE RDG: DID UPDATE: GRID: ', this.grid); // eslint-disable-line

        if (this.grid) {
            const canvas = this.grid.grid.querySelector('.react-grid-Canvas');
            console.log('DATA TABLE RDG: DID UPDATE: CANVAS: ', canvas); // eslint-disable-line
        }
    }

    /**
     * Handle drag and drop reordering of columns
     * @param {String} source
     * @param {String} target
     */
    handleHeaderDrop = (source, target) => {
        console.log('DATA TABLE RDG: HEADER DROP A: ', source, target); // eslint-disable-line

        const {
            columns,
        } = this.state;

        // Clone columns array
        const newColumns = [ ...columns ];

        // Get source and target column indexes
        const columnSourceIndex = columns.findIndex((i) => (i.key === source));
        const columnTargetIndex = columns.findIndex((i) => (i.key === target));

        // Reorder columns
        newColumns.splice(
            columnTargetIndex,
            0,
            newColumns.splice(columnSourceIndex, 1)[0]
        );

        // Need to modify column object due to 'immutable' use by RDG
        const reorderedColumns = newColumns.map((c, i) => {
            return {
                ...c,
                _reRenderKey: `${c.key}_${i}`,
            };
        });

        // Update columns
        this.setState({
            columns: reorderedColumns,
        });
    };

    rowGetter = (i) => {
        const {
            rows,
        } = this.state;
        return rows[i];
    };
}

export default DataTableRDG;
